# Generated by Django 3.2.16 on 2022-12-22 11:31

from django.db import migrations
import wagtailmarkdown.fields


class Migration(migrations.Migration):

    dependencies = [
        ('puput', '0005_blogpage_main_color'),
    ]

    operations = [
        migrations.AddField(
            model_name='entrypage',
            name='markdown_body',
            field=wagtailmarkdown.fields.MarkdownField(blank=True, null=True, verbose_name='body (Markdown)'),
        ),
    ]
